require "npg/project/version"
require 'shellwords'
require 'iniparse'
require 'FileUtils'
require 'zlib'
module NPG
	class Directory
		def [](*a)
			File.join @path, *a
		end
	end
	class PGHandle < Directory
		def initialize(path)
			@path = path
			unless FileTest.directory?(path)
				if FileTest.file?(path)
					raise Errno::EEXIST, path
				end
				Dir.mkdir path
			end
		end
	end
	class Project < Directory
		attr_accessor :path
		def initialize(path = nil)
			self.path = path
		end

		def path=(val)
			val = File.expand_path val
			if FileTest.directory?(val)
				@path = val
			elsif FileTest.file?(val)
				@path = File.dirname(val)
			else
				raise Error::ENOENT, val
			end
			val = val.encode("UTF-8")
		end

		def executable
			File.join(self.path, "Game.exe")
		end

		def handle
			PGHandle.new File.join self.path, ".npg"
		end

		def run(opt = {})
			opt = {
				exe:    executable,
				cmdline: nil,
				console: false,
				debug:   false,
			}.update(opt)
			npg = handle
			if code = opt[:code]
				code = code.encode("UTF-8")	
				FileUtils.cp self["Game.exe"], npg["Game.exe"]
				open npg["Game.ini"], 'w' do |f|
					f.write\
						%{[Game]
Scripts=npgldr
RTP=#{ini["Game"]["RTP"]}
RTP1=#{ini["Game"]["RTP1"]}
RTP2=#{ini["Game"]["RTP2"]}
RTP3=#{ini["Game"]["RTP3"]}
Title=#{ini["Game"]["Title"]}
Library=../#{ini["Game"]["Library"]}
}
				end
				realcode = [LDRHeader, %{
					NPG::StartupPath = Dir.getwd
					NPG::Path = #{path.inspect}
					NPG::RGSSScripts = #{ini["Game"]["Scripts"].inspect}
				}, code].join("\n\n") 
				x = [[0, "npgldr", Zlib::Deflate.deflate(realcode)]]
				open npg["npgldr"], 'w' do |f| Marshal.dump(x, f) end
				opt.update exe: npg["Game.exe"]
			end



			File.delete npg["error.txt"] if FileTest.file?(npg["error.txt"])
			cmdline = Shellwords.shelljoin([opt[:exe], opt[:cmdline], 
				   opt[:console] ? "console" : nil,
				   opt[:debug]   ? "debug": nil, 
				   opt[:debug]   ? "test": nil,
			].compact.map{|x| 
				Shellwords.escape(x)
			})
			system cmdline
			if FileTest.file?(npg["error.txt"])
				raise File.read(npg["error.txt"])
			end
		end

		LDRHeader = %{
     module NPG
        class << self
     	   def rgss_run
	    begin
	      Dir.chdir NPG::Path 
	      $RGSS_SCRIPTS = open(NPG::RGSSScripts, "rb") do |f| Marshal.load(f) end
	      $RGSS_SCRIPTS.each_with_index{|x, i|
	        x[3] = Zlib::Inflate.inflate(x[2])
	      }
	      type = (defined? msgbox) ? "{%04d}" : "Section%03d"
	      $RGSS_SCRIPTS.each_with_index{|x, i|
	        eval x[3], TOPLEVEL_BINDING, type % (i+1), 1
	      }
	    rescue Object
	      x = $!.backtrace.unshift($!.to_s).join("\n")
	      open(".npg/error.txt", "w") do |f|
                  f.write x
		  f.write "\n"
	      end
	    end
	   end
	end
     end



module Kernel
 alias oreq require
 def rb(code, args = nil)
  open "run.rb", "w" do |f|
    f.write "
      class Output
        def <<(obj)
          puts [Marshal.dump(obj)].pack('M')
        end
      end
      $output = Output.new
    begin
    "
    f.write code
    f.write "
  rescue LoadError
      $output << LoadError
    end
    "
  end
  system "ruby run.rb \#{args}> list.txt"
  Marshal.load File.read('list.txt').unpack("M").first
end

 private :rb
 $:.concat rb %{ $output << $: }
 def require(a)
   u = rb %{$output << Gem.find_files(ARGV[0])}, a
   raise u if u.is_a?(Exception)
   u.select(&FileTest.method(:file?)).any?{|req|oreq req}
 end
 
 Object::RUBY_VERSION = rb %{$output << RUBY_VERSION} 
 require 'rubygems'
 alias require oreq
end
     
		}

		def config_file
			File.join(self.path, "Game.ini")
		end

		def ini
			a = IniParse.parse File.read config_file
		end
	end

end

